@@ -14,8 +14,7 @@
 #
 # See the COPYING file for license information.
 #
-# Copyright (c) 2012 peo3 <peo314159265@gmail.com>
-
+# Copyright (c) 2012,2013 peo3 <peo314159265@gmail.com>
 from __future__ import with_statement
 import os
 import os.path
@@ -27,31 +26,25 @@ from cgutils import process
 
 class Command(command.Command):
     NAME = 'pgrep'
+    HELP = 'Search and show processes with cgroup like pgrep command'
     DEFAULT_SUBSYSTEM = 'cpu'
-    parser = command.Command.parser
-    parser.add_option('-o', action='store', type='string',
-                      dest='target_subsystem', default=DEFAULT_SUBSYSTEM,
-                      help='Specify a subsystem [%default]')
-    parser.add_option('-f', '--cmdline', action='store_true',
-                      dest='cmdline', default=False,
-                      help='Compare with entire cmdline of process')
-    parser.add_option('-l', '--show-name', action='store_true',
-                      dest='show_name', default=False,
-                      help='Show name of process')
-    parser.add_option('-i', '--ignore-case', action='store_true',
-                      dest='ignore_case', default=False,
-                      help='Ignore case')
-    parser.usage = "%%prog %s [options] <proc_name>" % NAME
 
-    def run(self, args):
-        if len(args) < 1:
-            self.parser.error('Less argument')
+    @staticmethod
+    def add_subparser(subparsers):
+        parser = subparsers.add_parser(Command.NAME, help=Command.HELP)
+        parser.add_argument('-o', action='store',
+                            dest='target_subsystem', default=Command.DEFAULT_SUBSYSTEM,
+                            help='Specify a subsystem [%(default)s]')
+        parser.add_argument('-f', '--cmdline', action='store_true',
+                            help='Compare with entire cmdline of process')
+        parser.add_argument('-l', '--show-name', action='store_true',
+                            help='Show name of process')
+        parser.add_argument('-i', '--ignore-case', action='store_true',
+                            help='Ignore case')
+        parser.add_argument('procname', metavar='PROCNAME', help='Process name')
 
-        if len(args) > 1:
-            self.parser.error('Too many arguments: ' + ' '.join(args))
-
-        procname = args[0]
-        root_cgroup = cgroup.scan_cgroups(self.options.target_subsystem)
+    def run(self):
+        root_cgroup = cgroup.scan_cgroups(self.args.target_subsystem)
 
         def print_matched(cg, dummy):
             mypid = os.getpid()
@@ -61,20 +54,20 @@ class Command(command.Command):
                     continue
                 proc = process.Process(pid)
 
-                if self.options.cmdline:
+                if self.args.cmdline:
                     comp = proc.cmdline
                 else:
                     comp = proc.name
 
-                if self.options.ignore_case:
+                if self.args.ignore_case:
                     comp = comp.lower()
-                    _procname = procname.lower()
+                    _procname = self.args.procname.lower()
                 else:
-                    _procname = procname
+                    _procname = self.args.procname
 
                 if _procname in comp:
-                    if self.options.show_name:
-                        if self.options.cmdline:
+                    if self.args.show_name:
+                        if self.args.cmdline:
                             output = "%d %s" % (proc.pid, proc.cmdline)
                         else:
                             output = "%d %s" % (proc.pid, proc.name)
