@@ -14,8 +14,7 @@
 #
 # See the COPYING file for license information.
 #
-# Copyright (c) 2012 peo3 <peo314159265@gmail.com>
-
+# Copyright (c) 2012,2013 peo3 <peo314159265@gmail.com>
 import sys
 
 from cgutils import cgroup
@@ -24,32 +23,31 @@ from cgutils import command
 
 class Command(command.Command):
     NAME = 'stats'
+    HELP = 'Show stats of cgroups'
     DEFAULT_SUBSYSTEM = 'cpu'
 
-    parser = command.Command.parser
-    parser.add_option('-o', action='store', type='string',
-                      dest='target_subsystem', default=DEFAULT_SUBSYSTEM,
-                      help='Specify a subsystem [%default]')
-    parser.add_option('-e', '--hide-empty', action='store_true',
-                      dest='hide_empty', default=False,
-                      help='Hide empty groups')
-    parser.add_option('-z', '--show-zero', action='store_true',
-                      dest='show_zero', default=False,
-                      help='Show zero values')
-    parser.add_option('-j', '--json', action='store_true',
-                      dest='json', default=False,
-                      help='Dump as JSON')
-    parser.usage = "%%prog %s [options]" % NAME
+    @staticmethod
+    def add_subparser(subparsers):
+        parser = subparsers.add_parser(Command.NAME, help=Command.HELP)
+        parser.add_argument('-o', action='store',
+                            dest='target_subsystem', default=Command.DEFAULT_SUBSYSTEM,
+                            help='Specify a subsystem [%(default)s]')
+        parser.add_argument('-e', '--hide-empty', action='store_true',
+                            help='Hide empty groups')
+        parser.add_argument('-z', '--show-zero', action='store_true',
+                            help='Show zero values')
+        parser.add_argument('-j', '--json', action='store_true',
+                            help='Dump as JSON')
 
     _INDENT = ' ' * 4
 
     def _print_stats(self, cgname, stats):
         def print_recursive(name, value, indent):
             if isinstance(value, long):
-                if self.options.show_zero or value != 0:
+                if self.args.show_zero or value != 0:
                     return "%s%s=%d\n" % (self._INDENT * indent, name, value)
             elif isinstance(value, list):
-                if self.options.show_zero or value:
+                if self.args.show_zero or value:
                     values = [str(v) for v in value]
                     return "%s%s=%s\n" % (self._INDENT * indent,
                                           name,
@@ -66,13 +64,13 @@ class Command(command.Command):
         if ret:
             print ret,
 
-    def run(self, args):
-        root_cgroup = cgroup.scan_cgroups(self.options.target_subsystem)
+    def run(self):
+        root_cgroup = cgroup.scan_cgroups(self.args.target_subsystem)
 
         def collect_configs(_cgroup, store):
-            if self.options.debug:
+            if self.args.debug:
                 print(_cgroup)
-            if self.options.hide_empty and _cgroup.n_procs == 0:
+            if self.args.hide_empty and _cgroup.n_procs == 0:
                 pass
             else:
                 store[_cgroup.path] = _cgroup.get_stats()
@@ -80,7 +78,7 @@ class Command(command.Command):
         cgroups = {}
         cgroup.walk_cgroups(root_cgroup, collect_configs, cgroups)
 
-        if self.options.json:
+        if self.args.json:
             import json
             json.dump(cgroups, sys.stdout, indent=4)
         else:
